/*
 * Copyright (C) 2010 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by
 *              Michal Hruby <michal.hruby@canonical.com>
 *
 */

#include "config.h"
#include <glib.h>
#include <glib-object.h>

#include <gtx.h>
#include <dee.h>

static void
_row_added (DeeModel *model, DeeModelIter *iter, GSList **added)
{
  /* Yes, I _know_ that append() is slow, but this is a test! */
  *added = g_slist_append (*added, iter);
}

/* Expects a clone with 3 rows in it */
gint
main (gint argc, gchar *argv[])
{
  DeeModel     *model;
  GSList       *added;
  
  g_type_init (); 
  g_thread_init (NULL);

  model = dee_shared_model_new (argv[1]);
  
  if (gtx_wait_for_signal (G_OBJECT (model), 100000, "notify::synchronized", NULL))
    g_error ("Helper model timed out waiting for 'ready' signal");

  g_assert_cmpint (dee_model_get_n_rows (model), ==, 3);

  /* Listen for adds */
  added = NULL;
  g_signal_connect (model, "row-added", G_CALLBACK (_row_added), &added);

  /* We expect the model to be cleared and 5 rows added */
  gtx_yield_main_loop (1000);

  /* The transaction could be optimized in the future and actually just add
   * the two missing rows */
  g_assert (g_slist_length (added) == 5 || g_slist_length (added) == 2);
  g_assert_cmpint (dee_model_get_n_rows (model), ==, 5);
  
  gtx_assert_last_unref (model);
  g_slist_free (added);
  
  return 0;
}
