/*
 * Copyright (C) 2011 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by Michal Hruby <michal.hruby@canonical.com>
 *
 */

#include <glib.h>
#include <glib-object.h>
#include <dee.h>
#include <gtx.h>

#define TIMEOUT 100
#define PEER_NAME "com.canonical.Dee.Peer.Tests.Interactions"

/* A command line that launches the appropriate peer-helper-* executable,
 * giving $name as first argument */
#define PEER_HELPER(helper,name) \
  (gchar *[]) { "./server-helper-"#helper, name, NULL }


typedef struct
{
  DeeServer *server;

} Fixture;

static void server_setup         (Fixture *fix, gconstpointer data);
static void server_teardown      (Fixture *fix, gconstpointer data);

static void test_allocation           (Fixture *fix, gconstpointer data);
static void test_valid_client_address (Fixture *fix, gconstpointer data);
static void test_client               (Fixture *fix, gconstpointer data);

void
test_client_server_interactions_create_suite (void)
{
#define DOMAIN "/ClientServer/Interactions"

  g_test_add (DOMAIN"/Allocation", Fixture, 0,
              server_setup, test_allocation, server_teardown);
  g_test_add (DOMAIN"/ValidClientAddress", Fixture, 0,
              server_setup, test_valid_client_address, server_teardown);
  g_test_add (DOMAIN"/OneClient", Fixture, 0,
              server_setup, test_client, server_teardown);
}

static void
server_setup (Fixture *fix, gconstpointer data)
{
  fix->server = dee_server_new (PEER_NAME);

  g_assert_cmpint (0, !=, dee_peer_is_swarm_leader (DEE_PEER (fix->server)));

  g_assert (DEE_IS_SERVER (fix->server));
}

static void
server_teardown (Fixture *fix, gconstpointer data)
{
  gtx_assert_last_unref (fix->server);

  /* Spin the mainloop a bit to check if we have any post-test
   * async effect crashing us */
  gtx_yield_main_loop (200);
}

static void
test_allocation (Fixture *fix, gconstpointer data)
{
  /* Do nothing, this test basically just asserts that
   * the fix->server is cleaned up after immediate construction */
}

static void
test_valid_client_address (Fixture *fix, gconstpointer data)
{
  const gchar *address = dee_server_get_client_address (fix->server);
  g_assert (address != NULL);
}

static void
test_client (Fixture *fix, gconstpointer data)
{
  /* We are leaders - launch the helper */
  if (gtx_wait_for_command (TESTDIR,
                            PEER_HELPER (client, PEER_NAME),
                            1000))
    g_critical ("Peer helper timed out");

  gtx_assert_last_command_status (0);
}
